/*
 * @Copyright 2010, MechSoft 
 * MechSoft, Mechanical and Software Solutions 
 * 
 * Licensed under the Apache License, Version 2.0 (the 
 * "License"); you may not use this file except in compliance 
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.sample.reservation.service;

import java.util.GregorianCalendar;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.apache.commons.validator.GenericValidator;
import org.defne.jpa.EntityManagerUtil;
import org.defne.service.Message;
import org.defne.service.MessageFactory;
import org.defne.service.annotation.Operation;
import org.defne.service.annotation.Service;
import org.defne.utility.Utility;
import org.defne.utility.exception.DefneException;
import org.defne.utility.log.ILogger;
import org.defne.utility.log.LoggerProvider;
import org.sample.reservation.entity.User;

@Service
public class LoginService
{
    private static final ILogger logger = LoggerProvider.getLogProvider(LoginService.class);
    
    public interface LOGIN_SERVICE
    {
        String SERVICE_NAME = "LoginService";
        interface CHECK_LOGIN_METHOD
        {
            String NAME = "checkLogin";
            String USER_NAME = "USER_NAME";
            String PASSWORD = "PASSWORD";
            String LOGIN_RESULT = "LOGIN_RESULT";
            String LOGIN_USER = "LOGIN_USER";
        }
        
    }

    @Operation
    public static Message checkLogin(Message serviceBag) throws DefneException
    {
        Message oBag = MessageFactory.newMessage();
        try
        {
            String userName = serviceBag.getMessageParameter(String.class, "USER_NAME");
            String password = serviceBag.getMessageParameter(String.class, "PASSWORD");

            if (GenericValidator.isBlankOrNull(userName) || GenericValidator.isBlankOrNull(password))
            {
                logger.warn("UserName and Password is null");
            }

            EntityManager entityManager = EntityManagerUtil.getEntityManagerFromBag(serviceBag);

            Query query = entityManager.createQuery("select p from User p where  " + "p.userName=:userName and " + "p.password=:password");

            query.setParameter("userName", userName);
            query.setParameter("password", password);

            User value = null;
            try
            {
                value = (User) query.getSingleResult();
                value.setLastLoginDate(GregorianCalendar.getInstance().getTime());
                
                oBag.putMessageParameter("LOGIN_RESULT", true);
                oBag.putMessageParameter("LOGIN_USER", value);

            }
            catch (Exception e)
            {
                oBag.putMessageParameter("LOGIN_RESULT", false);
            }
            
        }
        catch (Exception e)
        {
            Utility.throwsDefneException(e);
        }

        return oBag;
    }
}
