/*
 * @Copyright 2010, MechSoft 
 * MechSoft, Mechanical and Software Solutions 
 * 
 * Licensed under the Apache License, Version 2.0 (the 
 * "License"); you may not use this file except in compliance 
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.sample.reservation.service;

import javax.persistence.EntityManager;

import org.apache.commons.validator.GenericValidator;
import org.defne.jpa.EntityManagerUtil;
import org.defne.service.Message;
import org.defne.service.MessageFactory;
import org.defne.service.annotation.Operation;
import org.defne.service.annotation.Service;
import org.defne.utility.Utility;
import org.defne.utility.exception.DefneException;
import org.defne.utility.log.ILogger;
import org.defne.utility.log.LoggerProvider;
import org.sample.reservation.entity.User;
import org.sample.reservation.util.CalendarUtil;

@Service
public class RegisterService
{
    private static final ILogger logger = LoggerProvider.getLogProvider(RegisterService.class);
    
    @Operation
    public static Message registerUser(Message inBag) throws DefneException
    {
        Message oBag = MessageFactory.newServiceBag(inBag.isReturnsOutMessage());
        try
        {
            String userName = inBag.getMessageParameter(String.class, "USER_NAME");
            String password = inBag.getMessageParameter(String.class, "PASSWORD");
            String name = inBag.getMessageParameter(String.class, "NAME");
            String surname = inBag.getMessageParameter(String.class, "SURNAME");
            int age  = inBag.getMessageParameter(Integer.class, "AGE");
            boolean admin  = inBag.getMessageParameter(Boolean.class, "ADMIN");
            
            
            EntityManager entityManager = EntityManagerUtil.getEntityManagerFromBag(inBag);
            logger.info("Register a new user with user name : " + userName);
            
            if(GenericValidator.isBlankOrNull(userName) || GenericValidator.isBlankOrNull(password))
            {
                logger.info("Registering is failed. User name and password can not be null");
                return null;
            }
            
            User user = new User();
            
            user.setUserName(userName);
            user.setPassword(password);
            user.setName(name);
            user.setSurname(surname);
            user.setAge(age);
            user.setRegisterDate(CalendarUtil.getCurrentDate());
            user.setAdmin(admin);
            
            entityManager.persist(user);
            
            oBag.putMessageParameter("USER", user);
        }
        catch (Exception e)
        {
            Utility.throwsDefneException(e);
        }

        return oBag;
    }
}
